/*
  KeePT - Plugin for KeePass 1.21
  Copyright (C) 2010 Nikolaus Hammler <niki@hammler.net>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

// TODO: Make a proxy for _MVC functions
#define _CRT_SECURE_NO_WARNINGS

#include <tchar.h>
#include <assert.h>
#include "WinPTExec.h"
#include "KeePTPlugin.h"
#include "KeePTHotKeys.h"
#include "WinPT/src/resource.h"
#include "WinPTHooks.h"
#include <shlwapi.h>
#include <aclapi.h>

extern IKpAPI3 *g_pAPI;
extern HINSTANCE g_hDLL;
extern UINT WM_HKEYCNTRL;

//static HWND g_hLastWnd = NULL;

#ifdef _MSC_VER
#pragma data_seg(".SHARED")
HWND g_shhKeeWnd = NULL;
#pragma data_seg()
#pragma comment(linker, "/section:.SHARED,RWS")
#else
#define SHARED __attribute__((section(".SHARED"), shared)) 
HWND g_shhKeeWnd SHARED = NULL;
#endif

/**
 * @return: true: read password from shared memory. false: do not use KP
 */
bool QueryForGPGPassword(char *pwd, int maxlen, char *keyid)
{
	assert( g_pAPI );

	BOOL bLocked = g_pAPI->IsFileLocked();
	BOOL bOpen = g_pAPI->IsFileOpen();
	// The meaning:
	// (1) bLocked = FALSE, bOpen = TRUE: File loaded, workspace unlocked: read data, return true
	// (2) bLocked = TRUE, bOpen = FALSE: File loaded, workspace locked: Request to unlock
	// (3) bLocked = FALSE, bOpen = FALSE: No file loaded at all: Return false
	// (4) bLocked = TRUE, bOpen = TRUE: invalid
	if(bLocked == bOpen)
		return false;

	// Starting from here, only (1) and (2) possible
	if(bLocked)
	{
		HWND hForegroundWnd = GetForegroundWindow();
		g_pAPI->LockFile(FALSE);
		SetForegroundWindow(hForegroundWnd);
	}

	// continue only if we have case (1) now
	if(!(g_pAPI->IsFileLocked() == FALSE && g_pAPI->IsFileOpen() == TRUE))
		return false;

	IKpDatabase *pDb = NULL;
	if(g_pAPI->QueryInstance(SCLSID_KpDatabase, IID_IKpDatabase, (void**)&pDb) != S_OK)
		return false;

	char strFind[1024] = "GPG-Key-ID: 0x";
	strncat(strFind, keyid, 17);

	int count = pDb->GetEntryCount();
	for(int i=0; i < count; i++)
	{
		PW_ENTRY *pEntry = pDb->GetEntry(i);
		if(pEntry && pEntry->pszAdditional)
		{
			if(strstr(pEntry->pszAdditional, strFind))
			{
				pDb->UnlockEntryPassword(pEntry);
				strncpy(pwd, pEntry->pszPassword, maxlen);
				pDb->LockEntryPassword(pEntry);
				pDb->Release();
				return true;
			}
		}
	}

	char msg[1024];
	strcpy(msg, "The entry for the GPG key ID ");
	strncat(msg, keyid, 17);
	strcat(msg, " could not be found.\n(No entry with ");
	strncat(msg, strFind, 17+14);
	strcat(msg, " in comments field found)");

	MessageBox(g_pAPI->GetMainWindowHandle(), msg, MSGTITLE, MB_OK | MB_ICONWARNING);
	pDb->Release();
	return false;
}

/**
 * This code is taken from PuTTY
 */
PSID GetMySID()
{
	HANDLE proc = NULL, tok = NULL;
	TOKEN_USER *user = NULL;
	DWORD toklen, sidlen;
	PSID sid = NULL, ret = NULL;

	if((proc = OpenProcess(MAXIMUM_ALLOWED, FALSE, GetCurrentProcessId())) == NULL)
        goto cleanup;

	if(!OpenProcessToken(proc, TOKEN_QUERY, &tok))
		goto cleanup;

	if(!GetTokenInformation(tok, TokenUser, NULL, 0, &toklen) &&
		GetLastError() != ERROR_INSUFFICIENT_BUFFER)
		goto cleanup;

	if((user = (TOKEN_USER *)LocalAlloc(LPTR, toklen)) == NULL)
		goto cleanup;

	if(!GetTokenInformation(tok, TokenUser, user, toklen, &toklen))
		goto cleanup;

	sidlen = GetLengthSid(user->User.Sid);

	sid = (PSID)malloc(sidlen);

	if(!CopySid(sidlen, sid, user->User.Sid))
		goto cleanup;

	/* Success. Move sid into the return value slot, and null it out
	 * to stop the cleanup code freeing it. */
	ret = sid;
	sid = NULL;

cleanup:
	if(proc != NULL)
		CloseHandle(proc);
	if(tok != NULL)
		CloseHandle(tok);
	if(user != NULL)
		LocalFree(user);
	if(sid != NULL)
		free(sid);

	return ret;
}

LRESULT OnCopyData(HWND hwnd, HWND hSender, COPYDATASTRUCT *cds)
{
	LRESULT ret = FALSE;
	PSID mapowner = NULL;
	PSID ourself = NULL;
	HANDLE filemap = NULL;
	char *p = NULL;
	PSECURITY_DESCRIPTOR psd1 = NULL, psd2 = NULL;

	if(cds == NULL || cds->dwData != WM_GETPTPWD_MAGIC)
		return FALSE;

	// This method is mainly taken from PuTTY
	char *mapname = (char*)cds->lpData;

	if(mapname[cds->cbData - 1] != '\0')
		return FALSE;

	ourself = GetMySID();
	if(ourself == NULL)
		goto cleanup;

	filemap = OpenFileMapping(FILE_MAP_ALL_ACCESS, FALSE, mapname);
	if(filemap == NULL || filemap == INVALID_HANDLE_VALUE)
		goto cleanup;

	if(GetSecurityInfo(filemap, SE_KERNEL_OBJECT, OWNER_SECURITY_INFORMATION, &mapowner, NULL, NULL, NULL, &psd1) != ERROR_SUCCESS)
		goto cleanup;

	if(!EqualSid(mapowner, ourself))
		goto cleanup;

	p = (char*)MapViewOfFile(filemap, FILE_MAP_WRITE, 0, 0, 0);
	if(!p)
		goto cleanup;

	char keyid[WINPT_KEYID_MAXLEN];
	memset(keyid, 0x00, WINPT_KEYID_MAXLEN);
	strncpy(keyid, p, WINPT_KEYID_MAXLEN-1);
	if(QueryForGPGPassword(p, MAXPWDLEN-1, keyid))
		ret = WM_GETPTPWD_MAGIC;

cleanup:
	if(p)
		UnmapViewOfFile(p);
	if(filemap)
		CloseHandle(filemap);
	if(ourself)
		free(ourself);
	if(psd1)
		LocalFree(psd1);
	if(psd2)
		LocalFree(psd2);
	return ret;
}

/*BOOL OnKpmTrayNotify(LPARAM lParamW, LPARAM lParamL)
{
	// We save HWND of active forground window so we
	// can move it to foreground again afterwards (in the thread)
	//
	// Reason: Clicking on the tray icon looses focus of current
	// active window
	if(LOWORD(lParamL) == WM_MOUSEMOVE)
	{
		char x[1024];
		HWND hFgWnd = GetForegroundWindow();
		if(hFgWnd)
		{
			GetWindowText(hFgWnd, x, 1024-1);
			if(strlen(x) > 0 && g_hLastWnd != hFgWnd)
			{
				g_hLastWnd = hFgWnd;
			}
		}
	}
	return TRUE;
}*/

bool InstallPasswordHook(HANDLE hWinPTProcess)
{
	bool ret = false;
	DWORD hLibModule = 0;
	char libname[MAX_PATH] = "";
	void *pLibRemote = NULL;
	HANDLE hThread = NULL;

	memset(libname, 0, MAX_PATH);
	assert( g_hDLL != NULL );
	if(!GetModuleFileName(g_hDLL, libname, MAX_PATH-1))
		return false;

	pLibRemote = VirtualAllocEx(hWinPTProcess, NULL, sizeof(libname), MEM_COMMIT, PAGE_READWRITE);
	if(!pLibRemote)
		goto cleanup;

	if(!WriteProcessMemory(hWinPTProcess, pLibRemote, (void*)libname, sizeof(libname), NULL))
		goto cleanup;

	hThread = CreateRemoteThread(hWinPTProcess, NULL, 0, (LPTHREAD_START_ROUTINE)GetProcAddress(GetModuleHandle(_T("kernel32")), _T("LoadLibraryA")), pLibRemote, 0, NULL);
	if(!hThread)
		goto cleanup;

	if(WaitForSingleObject(hThread, INFINITE) == WAIT_FAILED)
		goto cleanup;
		
	if(!GetExitCodeThread(hThread, &hLibModule) || hLibModule == 0)
		goto cleanup;

	ret = true;

cleanup:
	if(hThread)
		CloseHandle(hThread);
	if(pLibRemote)
		VirtualFreeEx(hWinPTProcess, pLibRemote, sizeof(libname), MEM_RELEASE);
	return ret;
}

void SendHotKeyCommand(int cmd)
{
	if(WM_HKEYCNTRL)
		SendMessage(g_pAPI->GetMainWindowHandle(), WM_HKEYCNTRL, cmd, 0);
}

/**
 * @param szPath: string containing path to GnuPT-Portable.exe
 * @param maxPathLen: buffer length of szPath (including terminating 0)
 */
void GenerateProtableMsgWarning(const char *szPath, int maxPathLen)
{
	char strIni[] = "\\GnuPT-Portable.ini";
	char *strIniPath = new char[maxPathLen + sizeof(strIni)];
	
	memset(strIniPath, 0x00, maxPathLen + sizeof(strIni));
	strncpy(strIniPath, szPath, maxPathLen - 1);
	PathRemoveFileSpec(strIniPath);
	strncat(strIniPath, strIni, sizeof(strIni) - 1);
	HANDLE hFile = CreateFile(strIniPath, GENERIC_READ, 0, NULL, OPEN_EXISTING, 0, NULL);
	delete strIniPath;strIniPath = NULL;
	if(hFile != INVALID_HANDLE_VALUE)
	{
		char *buf = new char[4096];
		memset(buf, 0x00, sizeof(buf));
		DWORD dwRead = 0;
		if(ReadFile(hFile, buf, sizeof(buf) - 1, &dwRead, NULL))
		{
			if(strstr(buf, "Message=1"))
			{
				assert( g_pAPI );
				MessageBox(g_pAPI->GetMainWindowHandle(),
					_T("GnuPT-Portable is set with to display messages (Message=1 in GnuPT-Portable.ini).\n"
					"This does not work reliable with KeePT. Please consider to turn off this feature."), MSGTITLE, MB_OK | MB_ICONWARNING);
			}
		}
		delete buf;buf = NULL;
		CloseHandle(hFile);
	}
}



DWORD WINAPI WinPTDispatchThread(LPVOID lpParam)
{
#ifdef _MSC_VER
#pragma warning(disable:4311)
#endif
	DWORD code = (DWORD)lpParam;
#ifdef _MSC_VER
#pragma warning(default:4311)
#endif
	BOOL bStarted = FALSE;
	BOOL bClose = TRUE;
	BOOL bPortable = FALSE;
	// memory: SI_REGSIZE + strlen(" -WinPT")
	char szPath[SI_REGSIZE + 10] = _T("WinPT.exe");
	IKpConfig *pCfg = NULL;
	HANDLE hWinPTProcess = NULL;

	if(g_pAPI->CreateInstance(CLSID_KpConfig_ReadOnly, IID_IKpConfig, (void**)&pCfg) != S_OK)
		return -1;

	bClose = pCfg->GetBool(CONF_WPTCLOSE, CONF_WPTCLOSE_DEF);
	bPortable = pCfg->GetBool(CONF_WPTUSEPORTABLE, CONF_WPTUSEPORTABLE_DEF);
	if(bPortable)
		strcpy(szPath, "GnuPT-Portable.exe");
	else
		strcpy(szPath, "WinPT.exe");
	pCfg->Get(CONF_WPTPATH, szPath);
	if(bPortable)
		strcat(szPath, " -WinPT");
	pCfg->Release();

	HWND hPTwnd = FindWindow(WPT_WNDTITLE, WPT_WNDCLASS);

	// No WinPT window found, start process ...
	if(!hPTwnd)
	{
		PROCESS_INFORMATION pi;
		STARTUPINFO si;
		ZeroMemory(&pi, sizeof(pi));
		ZeroMemory(&si, sizeof(si));

		si.cb = sizeof(si);

		SendHotKeyCommand(WM_HKEYCNTRL_UNREGISTER);

		if(bPortable)
		{
			GenerateProtableMsgWarning(szPath, sizeof(szPath));
		}

		if(CreateProcess(NULL, szPath, NULL, NULL, FALSE, 0, NULL, NULL, &si, &pi) == 0)
		{
			MessageBox(g_pAPI->GetMainWindowHandle(), _T("Unable to execute WinPT"), MSGTITLE, MB_OK | MB_ICONERROR);
			SendHotKeyCommand(WM_HKEYCNTRL_UPDATE);
			return -2;
		}

		CloseHandle(pi.hThread);

		/*if(WaitForInputIdle(pi.hProcess, PTEXCTIMEOUT) != 0)
		{
			CloseHandle(pi.hProcess);
			MessageBox(g_pAPI->GetMainWindowHandle(), _T("Waiting time for WinPT exceeded"), MSGTITLE, MB_OK | MB_ICONERROR);
			SendHotKeyCommand(WM_HKEYCNTRL_UPDATE);
			return -3;
		}*/
		WaitForInputIdle(pi.hProcess, INFINITE);

		CloseHandle(pi.hProcess);

		// wait for 20s for portable and 3s for non-portable max.
		int ttl = bPortable ? 80 : 12;
		for(int i = 0; i < ttl; i++)
		{
			hPTwnd = FindWindow(WPT_WNDTITLE, WPT_WNDTITLE);
			if(hPTwnd)
				break;
			Sleep(250);
		}

		if(!hPTwnd)
		{
			if(bPortable)
				MessageBox(g_pAPI->GetMainWindowHandle(), _T("WinPT did not start up within 20 seconds, giving up..."), MSGTITLE, MB_OK | MB_ICONERROR);
			else
				MessageBox(g_pAPI->GetMainWindowHandle(), _T("WinPT did not start up within 3 seconds, giving up..."), MSGTITLE, MB_OK | MB_ICONERROR);
			SendHotKeyCommand(WM_HKEYCNTRL_UPDATE);
			return -4;
		}
		
		bStarted = TRUE;
	}

	DWORD pid;
	GetWindowThreadProcessId(hPTwnd, &pid);
	hWinPTProcess = OpenProcess(PROCESS_ALL_ACCESS, FALSE, pid);

	WaitForInputIdle(hWinPTProcess, INFINITE);

	// TODO: Only hook instance if it is not yet hooked. This could be
	// achieved, that the DLL creates a Mutex with name "KeePT_<PID>"
	// which is queried here.
	// However, currently, it seems to work without that
	if(!InstallPasswordHook(hWinPTProcess))
		MessageBox(GetDesktopWindow(), _T("KeePT was not able to hook on WinPT.exe. KeePT is not able to pass the GPG password"), MSGTITLE, MB_OK | MB_ICONWARNING);

	WaitForInputIdle(hWinPTProcess, INFINITE);

	// Still we need to wait for startup (until WinPT has loaded the keys)
	// otherwise WinPT crashes :-(
	// TODO: FIXXME: Find a reliable condition on how WinPT is ready
	Sleep(500);

	WaitForInputIdle(hWinPTProcess, INFINITE);
	CloseHandle(hWinPTProcess);

	// TODO: use a Mutex for pathological case
	g_shhKeeWnd = g_pAPI->GetMainWindowHandle();

//	SetForegroundWindow(g_hLastWnd);
	SetForegroundWindow(hPTwnd);
	SendMessage(hPTwnd, WM_COMMAND, code, 0);

	if(bStarted && bClose)
		SendMessage(hPTwnd, WM_DESTROY, 0, 0);

	if(WM_HKEYCNTRL && bStarted)
		SendMessage(g_pAPI->GetMainWindowHandle(), WM_HKEYCNTRL, WM_HKEYCNTRL_UPDATE, 0);

	return 0;
}

bool ExecuteWinPT(DWORD commandID)
{
	DWORD threadID;
#ifdef _MSC_VER
#pragma warning(disable:4312)
#endif
	if(CreateThread(NULL, 0, WinPTDispatchThread, (LPVOID)commandID, 0, &threadID) == NULL)
		return false;
#ifdef _MSC_VER
#pragma warning(disable:4312)
#endif
	return true;
}

bool ShowWinPTTrayMenu()
{
	IKpConfig *pCnf = NULL;
	HINSTANCE hDll = NULL;

	if(g_pAPI->CreateInstance(CLSID_KpConfig, IID_IKpConfig, (void**)&pCnf) != S_OK)
		return false;

	char szPath[SI_REGSIZE + MAX_PATH] = ""; // include \\WinPT\\WinPT.exe
	if(pCnf->Get(CONF_WPTPATH, szPath))
	{
		// For portable version, we assume that WinPT.exe is in
		// ./WinPT/WinPT.exe
		if(pCnf->GetBool(CONF_WPTUSEPORTABLE, CONF_WPTUSEPORTABLE_DEF))
		{
			PathRemoveFileSpec(szPath);
			strcat(szPath, "\\WinPT\\WinPT.exe");
		}
		// we have plain WinPT
		hDll = LoadLibrary(szPath);
	}
	else
	{
		if(pCnf->GetBool(CONF_WPTUSEPORTABLE, CONF_WPTUSEPORTABLE_DEF))
		{
			hDll = LoadLibrary(_T("WinPT\\WinPT.exe"));
		}
		else
		{
			hDll = LoadLibrary(_T("WinPT.exe"));
		}
	}
	pCnf->Release();

	if(!hDll)
	{
		return false;
	}

	HMENU hMenu = LoadMenu(hDll, MAKEINTRESOURCE(IDR_WINPT));
	FreeLibrary(hDll);

	if(!hMenu)
		return false;

	HMENU hPopup = GetSubMenu(hMenu, 0);
	if(!hPopup)
		return false;

	MENUITEMINFO item;
	ZeroMemory(&item, sizeof(MENUITEMINFO));
	item.cbSize = sizeof(MENUITEMINFO);

	item.fMask = MIIM_ID;
	item.wID = 1;

	item.fMask |= MIIM_STRING;
	item.dwTypeData = (LPSTR)_T("KeePT Settings");

	InsertMenuItem(hPopup, 0, TRUE, &item);

	// insert separator
	item.fMask = 0;
	item.wID = 0;
	item.fMask = MIIM_TYPE;
	item.fType = MFT_SEPARATOR;
	InsertMenuItem(hPopup, 1, TRUE, &item);


	POINT p;
	GetCursorPos(&p);
	DWORD dwRetCode = TrackPopupMenu(hPopup, TPM_NONOTIFY|TPM_RETURNCMD|TPM_LEFTALIGN, p.x, p.y, 0,
		g_pAPI->GetMainWindowHandle(),
		NULL);

	if(dwRetCode)
	{
		if(dwRetCode == 1)
		{
			KeePTSettings();
			return true;
		}
		return ExecuteWinPT(dwRetCode);
	}
	return true;
}


